<?php
namespace Teknisa\MAN\Controllers;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Teknisa\MAN\Services\RelDREService;
use Teknisa\MAN\Services\RelDREComparativoService;
use Teknisa\MAN\Services;
use Teknisa\MAN\Utilities;
use Zeedhi\Framework\Controller\CrudWithParameterBag;
use Zeedhi\Framework\Report\ReportService;

class RelDRE  extends  CrudWithParameterBag { 
    
    const dataSourceClassFina   = 'select_classFina';
    const dataSourceParamGeral  = 'select_paramsGeraisDRE';
    const dataSourcePlanoContas = 'select_planoContas';
    const MENSAGEM = "Até 36 filiais por geração!";
    const OPERADOR = "Código do operador vazio! DRE ñ gera com este código vazio.";
    
    protected $utilService,
              $parameterBagColumns = array('NRORG');
    protected $formatBirt;
              
    public function __construct ($crudService, $parameterBag, $entityManager, Utilities\Util $utilService, Manager $dataSourceManager,
                                 RelDREService $RelDREService, RelDREComparativoService $DREC_Service, BirtReport $birtReport, ReportService $reportService/*, birtStrategy $birtStrategy*/) {
        $this->crudService          = $crudService;
        $this->parameterBag         = $parameterBag;
        $this->entityManager        = $entityManager;
        $this->utilService          = $utilService;
        $this->dataSourceManager    = $dataSourceManager;
        $this->DREService           = $RelDREService;
        $this->DREC_Service         = $DREC_Service;
        $this->birtReport           = $birtReport;
        $this->reportService        = $reportService;
        // $this->birtStrategy         = $birtStrategy;
    }
    
   /*********************
    *   PARAM METHODS   *
    *********************/
    
    public function findClassFina(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceClassFina;
        $this->parameterBagColumns = $this->parameterBagColumns;
        parent::find($request,$response);
    }
    
    public function findParamGerais(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceParamGeral;
        $this->parameterBagColumns = $this->parameterBagColumns;
        parent::find($request,$response);
    }
    
    public function findPlanoContas(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourcePlanoContas;
        $this->parameterBagColumns = $this->parameterBagColumns;
        parent::find($request,$response);
    }
    
    public function saveGeralTab(DTO\Request\Row $request, DTO\Response $response) {
        $conditions = $this->utilService->getParamsAsArray($request);
        $resposta   = $this->DREService->saveGeralTab($conditions);
        return $resposta;
    }
    
    public function saveClassFina(DTO\Request\Row $request, DTO\Response $response) {
        $conditions = $this->utilService->getParamsAsArray($request);
        $verificacao = $this->DREService->verifyClassFina($conditions);
        if($verificacao[0] == true){
            $resposta = $this->DREService->saveClassFina($conditions);
            return $resposta;
        } else {
            $response->setError(new Error('O valor de ordem: '.$verificacao[2].' já foi registrado em '.$verificacao[1], 500));
        }
    }
    
    public function saveOrdemInLine(DTO\Request\DataSet $request, DTO\Response $response) {
        $rows = $request->getDataSet()->getRows();
        foreach($rows as $row){
            $params = array(
                'IDORDEMDRE'    => $row['IDORDEMDRE_inline'],
                'CDCLASSFINA'   => $row['CDCLASSFINA'],
                'NRORG'         => $row['NRORG']
            );
            $verificacao = $this->DREService->verifyClassFina($params);
            if($verificacao[0] == true){
                $conditions = array(
                    'CDCLASSFINA'   => $row['CDCLASSFINA'],
                    'NRORG'         => $row['NRORG'],
                    'IDTPCLASS'     => $row['IDTPCLASS'],
                    'IDVISUDRE'     => $row['IDVISUDRE'],
                    'IDORDEMDRE'    => $row['IDORDEMDRE_inline']
                );
                $resposta = $this->DREService->saveClassFina($conditions);
                $conditions = array();
            } else {
                $response->setError(new Error('O valor de ordem: '.$verificacao[2].' já foi registrado em '.$verificacao[1], 500));
            }
        }
    }
    
    
    
    
    /*******************
     *  REPORT METHODS *
     *******************/
     
     
    public function findRelatorio(DTO\Request\Row $request, DTO\Response $response) {
        $conditions = $this->utilService->getParamsAsArray($request);
        $params     = $this->DREService->verifyReportType($conditions);
        if(empty($conditions["CDOPERADOR"])){
            $response->addMessage(new Message(self::OPERADOR, Message::TYPE_WARNING));
        }else{
            if(!isset($conditions['FORMAT'])){
    		    $this->formatBirt = 'pdf';
        	}else{
        		$this->formatBirt = $conditions['FORMAT'];
        	}
        	$this->getReport($params, $response);
            // $this->birtStrategy->setReportFormat($row["FORMAT"]);
        }
    }
    
    public function findRelatorioComparativo(DTO\Request\Row $request, DTO\Response $response){
        $conditions = $this->utilService->getParamsAsArray($request);
        $params     = $this->DREC_Service->mensalParamBuild($conditions);
        if(empty($conditions["CDOPERADOR"])){
            $response->addMessage(new Message(self::OPERADOR, Message::TYPE_WARNING));
        }else{
            if(!isset($conditions['FORMAT'])){
                $this->formatBirt = 'pdf';
            }else{
                $this->formatBirt = $conditions['FORMAT'];
            }
            $this->getReport($params, $response);
            // $this->birtStrategy->setReportFormat($row["FORMAT"]);
        }
    }
    
    public function findRelatorioComparativoUnidade(DTO\Request\Row $request, DTO\Response $response){
        $conditions = $this->utilService->getParamsAsArray($request);
        $expFiliais = $this->DREC_Service->expFiliais(count($conditions['CDFILIAL']));
        if(empty($conditions["CDOPERADOR"])){
            $response->addMessage(new Message(self::OPERADOR, Message::TYPE_WARNING));
        }else{
            if($expFiliais){
                $response->addMessage(new Message(self::MENSAGEM, Message::TYPE_WARNING));
            }else {
                if(!isset($conditions['FORMAT'])){
                    $this->formatBirt = 'pdf';
                }else{
                    $this->formatBirt = $conditions['FORMAT'];
                }
                $params     = $this->DREC_Service->UnidadeParamBuild($conditions);
                $this->getReport($params, $response);
                // $this->birtStrategy->setReportFormat($row["FORMAT"]);
            }
        }
    }

    public function getReport($params, DTO\Response $response){
        try {
            $birtURL = $this->reportService->createRemoteReport('BIRT', $params['ReportName'], $params['ParametersRow']);
            if ($this->formatBirt === 'xls') {
                $birtURL = str_replace('pdf', 'xls', $birtURL);
            }
            $response->addDataset(new DataSet('URL',array('url'=>$birtURL)));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage() , 500));
        }
    }
    
    
}